@@ -6,6 +6,8 @@ use warnings FATAL => 'all';
 use Carp;
 use LWP::UserAgent;
 use LWP::Protocol::https;
+use HTTP::Message;
+use XML::Simple;
 
 =head1 NAME
 
@@ -13,11 +15,11 @@ Net::Amazon::Utils - Implementation of a set of utilities to help in developing
 
 =head1 VERSION
 
-Version 0.01
+Version 0.02
 
 =cut
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 =head1 SYNOPSIS
 
@@ -83,7 +85,7 @@ sub new {
 	$no_cache = 0 unless defined $no_cache;
 
 	my $self = {
-		remote_region_file => 'http://raw.githubusercontent.com/aws/aws-sdk-android-v2/master/src/com/amazonaws/regions/regions.xml',
+		remote_region_file => 'https://raw.githubusercontent.com/aws/aws-sdk-android-v2/master/src/com/amazonaws/regions/regions.xml',
 		# do not cache regions between calls, does not affect Internet caching, defaults to false.
 		no_cache => $no_cache,
 		# do not load updated file from the Internet, defaults to true.
@@ -504,7 +506,8 @@ sub _load_regions {
 	my ( $self, $force ) = @_;
 
 	if ( $force || !defined $self->{regions} ) {
-		my @xml_options = [ KeyAttr => {Region => 'Name', Endpoint=>'ServiceName', Service => 'Name' } ];
+		my $error;
+
 		my $new_regions;
 		if ( $self->{no_inet} ) {
 			eval {
@@ -513,53 +516,61 @@ sub _load_regions {
 			};
 			if ( $@ ) {
 				carp "Processing XML failed with error $@";
+				$error = 1;
 			}
 		} else {
-			my $error;
-			my $response = $self->{ua}->get( $self->{remote_region_file} );
+			my $response = $self->{ua}->get( $self->{remote_region_file},
+																				'Accept-Encoding' => scalar HTTP::Message::decodable,
+																				'If-None-Match' => $self->{region_etag} );
 			if ( $response->is_success ) {
+				# Store etag for later tests
+				$self->{region_etag} = $response->header( 'Etag' );
 				# This should be a big file...
-				carp "Size of region file looks really suspicious." if ( length $response->decoded_content < 10000 );
+				my $content = $response->decoded_content;
+				carp "Size of region file looks suspiciously small." if ( length $content < 10000 );
 				eval {
-					$new_regions = XML::Simple::XMLin( $response->decoded_content, @xml_options );
+					my @xml_options = ( KeyAttr => { Region => 'Name', Endpoint=>'ServiceName', Service => 'Name' } );
+					$new_regions = XML::Simple::XMLin( $content, @xml_options );
+
+					# Check that some "trustable" regions and services exist.
+					unless ( defined $new_regions &&
+						defined $new_regions->{Regions} &&
+						defined $new_regions->{Regions}->{Region}->{'us-east-1'} &&
+						defined $new_regions->{Regions}->{Region}->{'us-west-1'} &&
+						defined $new_regions->{Regions}->{Region}->{'us-west-2'} &&
+						defined $new_regions->{Services} &&
+						defined $new_regions->{Services}->{Service}->{ec2} &&
+						defined $new_regions->{Services}->{Service}->{sqs} &&
+						defined $new_regions->{Services}->{Service}->{glacier}
+					) {
+						croak "Region file format cannot be trusted.";
+					}
 				};
 				if ( $@ ) {
 					carp "Processing XML failed with error $@";
 					$error = 1;
 				}
 			} else {
-				carp "Getting updated regions failed with " . $response->status_line;
-				$error = 1;
-			}
-			# Retry locally on errors
-			if ( $error ) {
-				my $old_no_inet = $self->{no_inet};
-				carp "Getting regions file from Internet failed will use local cache. Check your Internet connection...";
-				$self->{no_inet} = 1;
-				$self->_load_regions();
-				$self->{no_inet} = $old_no_inet
+				unless ( $response->code() eq '304' ) {
+					carp "Getting updated regions failed with " . $response->status_line;
+					$error = 1;
+				}
 			}
 		}
-		# Check that some "trustable" regions and services exist.
-		if ( defined $new_regions &&
-					defined $new_regions->{Regions} &&
-					defined $new_regions->{Regions}->{Region}->{'us-east-1'} &&
-					defined $new_regions->{Regions}->{Region}->{'us-west-1'} &&
-					defined $new_regions->{Regions}->{Region}->{'us-west-2'} &&
-					defined $new_regions->{Services} &&
-					defined $new_regions->{Services}->{Service}->{ec2} &&
-					defined $new_regions->{Services}->{Service}->{sqs} &&
-					defined $new_regions->{Services}->{Service}->{glacier}
-		) {
-			$new_regions->{Regions} = $new_regions->{Regions}->{Region};
-			$new_regions->{Services} = $new_regions->{Services}->{Service};
+		# Retry locally on errors
+		if ( $error ) {
+			my $old_no_inet = $self->{no_inet};
+			carp "Getting regions file from Internet failed will use local cache. Check your Internet connection...";
+			$self->{no_inet} = 1;
+			$self->_load_regions();
+			$self->{no_inet} = $old_no_inet
+		}
+		$new_regions->{Regions} = $new_regions->{Regions}->{Region};
+		$new_regions->{Services} = $new_regions->{Services}->{Service};
 
-			$self->{regions} = $new_regions if ( defined $new_regions );
-			# Create a set of correct protocols for this set
-			$self->reset_known_protocols();
-		} else {
-			croak "Region file format cannot be trusted.";
-		}
+		$self->{regions} = $new_regions if ( defined $new_regions );
+		# Create a set of correct protocols for this set
+		$self->reset_known_protocols();
 	}
 }
 
